#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Control/Item.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: 

package SAPDB::Install::Control::Item;

use Wx::Event qw (
	EVT_RIGHT_DOWN 
	EVT_SET_FOCUS EVT_PAINT);

use Wx qw (wxICON_ERROR wxEVT_COMMAND_BUTTON_CLICKED
			wxCOPY wxTRANSPARENT wxWHITE wxSOLID wxNullColour
			wxTRANSPARENT_BRUSH wxSYS_SYSTEM_FONT wxSYS_DEFAULT_GUI_FONT);

use base Wx::PlWindow;

our $hgap = 5;

sub new{
	my($self ) = shift->SUPER::new(@_);
	$self->SetBackgroundColour($self->GetParent->GetBackgroundColour);
	$self->{'selected'} = 0;
	EVT_RIGHT_DOWN($self,\&OnRightDown);
	$self->SetNextHandler($self->GetParent);
	EVT_SET_FOCUS($self,\&OnSetFocus);
	EVT_PAINT($self,\&OnPaint);
	return $self;
}

sub SetLabel{
	my ($self,$label) = @_;
	if	(not exists $self->{'label'} or 
			$self->{'label'} ne $label){
		$self->{'label'} = $label;
		$self->{'text_changed'} = 1;
		$self->Render;
		$self->Refresh;
		$self->Update;
	}
	return 1;	
}

sub ShowContextMenu{
	my ($self,$pos) = @_;
}	

sub SetImage{
	my ($self,$image) = @_;
	$self->{'image'} = $image;
	$self->{'image_changed'} = 1;
	$self->Render;
	$self->Refresh;
	$self->Update;
	return 1;	
}



sub RenderImage{
	my ($self) = @_;
	return undef unless exists $self->{'image'};
	unless (exists $self->{'image_mdc'}){
		$self->{'image_mdc'} = Wx::MemoryDC->new();
	}
	$self->{'image_mdc'}->SelectObject(Wx::Bitmap->new($self->{'image'}));
	if (exists $self->{'image_changed'}){
		delete $self->{'image_changed'};
	}
}

sub RenderText{
	my ($self) = @_;
	return undef unless exists $self->{'label'};
	my $dummy_dc = Wx::MemoryDC->new();
	unless ($^O =~ /mswin/i){
		$dummy_dc->SelectObject(Wx::Bitmap->new(1,1));
	}
	unless (exists $self->{'text_mdc'}){
		$self->{'text_mdc'} = Wx::MemoryDC->new();
		if($^O =~ /mswin/i){
			$self->{'text_mdc'}->SetFont(Wx::SystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
		}
		else{
			$self->{'text_mdc'}->SetFont(Wx::SystemSettings::GetFont(wxSYS_SYSTEM_FONT));
		}
		if($^O !~ /mswin/i){
			$self->{'text_mdc'}->SelectObject(Wx::Bitmap->new(1,1));
		}
	}
	my ($text_w,$text_h) = $self->{'text_mdc'}->GetTextExtent($self->{'label'});
	$self->{'text_mdc'}->BeginDrawing() if $^O =~ /mswin/i;
	$self->{'text_mdc'}->SelectObject(Wx::Bitmap->new($text_w,$text_h));
	$self->{'text_mdc'}->SetBackground(Wx::Brush->new($self->GetBackgroundColour,wxSOLID));
	$self->{'text_mdc'}->Clear;
	$self->{'text_mdc'}->DrawText($self->{'label'},0,0);
	if (exists $self->{'text_changed'}){
		delete $self->{'text_changed'};
	}
	$self->{'text_mdc'}->EndDrawing() if $^O =~ /mswin/i;
	return 1;
}

sub Render{
	my ($self) = @_;
	if ( not exists $self->{'image_mdc'} or exists $self->{'image_changed'}){
		$self->RenderImage;
	}
	if (not exists $self->{'text_mdc'} or exists $self->{'text_changed'}){
		$self->RenderText;
	}
	
	if (not exists $self->{'text_mdc'} and not exists $self->{'image_mdc'}){
		return undef;
	}
		
	my $w = 0;
	$w = $self->{'image_mdc'}->GetSize->GetWidth() if exists $self->{'image_mdc'}; 
	my $imagew = $w;
	my $h = 0; 
	$h = $self->{'image_mdc'}->GetSize->GetHeight() if exists $self->{'image_mdc'};
	$w += 	$self->{'text_mdc'}->GetSize->GetWidth() if (exists $self->{'text_mdc'});
	$w += $hgap * 2;
	$h = $self->{'text_mdc'}->GetSize->GetHeight()
		if exists $self->{'text_mdc'} and 
		$self->{'text_mdc'}->GetSize->GetHeight() > $h;
	unless (exists $self->{'mdc'}){
		$self->{'mdc'} = Wx::MemoryDC->new();
	}
	$self->SetSize($w,$h) unless exists $self->{'no_autoresize'};
	$self->{'mdc'}->BeginDrawing() if $^O =~ /mswin/i;
	$self->{'mdc'}->SelectObject(Wx::Bitmap->new($w,$h));
	$self->{'mdc'}->SetBackground(Wx::Brush->new($self->GetBackgroundColour,wxSOLID));
	$self->{'mdc'}->Clear();
	if (exists $self->{'image_mdc'}){
		my $ypos = ($h - $self->{'image_mdc'}->GetSize->GetHeight());	
		$ypos = 0 if $ypos < 0;
		$self->{'mdc'}->Blit(0,$ypos,
					$self->{'image_mdc'}->GetSizeWH,
					$self->{'image_mdc'},0,0,
					wxCOPY,
					1);
	}
	if (exists $self->{'text_mdc'}){
		my $ypos = ($h - $self->{'text_mdc'}->GetSize->GetHeight());	
		$ypos = 0 if $ypos < 0;
		$self->{'mdc'}->Blit($imagew + $vgap,$ypos,
				   $self->{'text_mdc'}->GetSizeWH,
				   $self->{'text_mdc'},0,0,
				   wxCOPY,
				   1);
	}
	$self->{'mdc'}->EndDrawing() if $^O =~ /mswin/i;		   	
}



sub OnPaint{
	my ($self,$evt) = @_;
	return unless defined $self->{'mdc'};	
	my $pdc = Wx::PaintDC->new($self);
	if (exists $self->{'no_autoresize'}){
		$pdc->SetBackground(Wx::Brush->new(
				$self->GetBackgroundColour,wxSOLID));	
		$pdc->Clear();
	}
	$pdc->Blit(0,0,
			   $self->{'mdc'}->GetSizeWH,
			   $self->{'mdc'},0,0,
			   wxCOPY,
			   1);
	$evt->Skip;
}

sub OnRightDown{
	my ($self,$evt) = @_;
	$self->ShowContextMenu($evt->GetPosition);
}

sub OnSetFocus{
	my ($self,$event) = @_;
}

sub Destroy{
	my ($self) = @_;
	if (exists $self->{'image'}){
		delete $self->{'image'};
	}
	if (exists $self->{'mdc'}){
		delete $self->{'mdc'};
	}
	if (exists $self->{'text_mdc'}){
		delete $self->{'text_mdc'};
	}
	if (exists $self->{'image_mdc'}){
		delete $self->{'image_mdc'};
	}
	$self->SUPER::Destroy;

}

sub DESTROY{
	my ($self) = @_;
	print ">>>> DESTROY $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}



1;