#!/usr/bin/perl
#
# $Header: //sapdb/I00/develop/sys/src/install/perl/SAPDB/Install/Control/StaticText.pm#1 $
# $DateTime: 2003/08/01 16:13:03 $
# $Change: 49268 $
#
# Desc: 

package SAPDB::Install::Control::StaticImage;

use Wx::Event qw (EVT_PAINT);
use Wx qw (wxBLACK wxSOLID);

use base SAPDB::Install::Helpers::PassiveWin;

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->{'brush'} = Wx::Brush->new($this->GetParent()->GetBackgroundColour(),wxSOLID);
	$this->SetBackgroundColour($this->GetParent->GetBackgroundColour);
	$this->{'pen'} = Wx::Pen->new(wxBLACK,1,wxSOLID);
	EVT_PAINT($this,\&OnPaint);
	return $this;
}


sub SetBitmap{
	my ($self,$bmp) = @_;
	$self->{'bmp'} = $bmp;
	$self->SetSize($bmp->GetWidth,$bmp->GetHeight);
	$self->Refresh;
	$self->Update;
}

sub SetData{
	my ($self,$img_data) = @_;
	$self->{'data'} = $img_data->[0];
	$self->{'size'} = [$img_data->[1],$img_data->[2]];
	$self->SetSize($img_data->[1],$img_data->[2]);
	$self->Refresh;
	$self->Update;
}

sub SetTransColour(){
	my ($self,$rgb) = @_;
	$self->{'trans_colour'} = $rgb;
}

sub SetMDC{
	my ($self,$mdc) = @_;
	return unless defined $mdc; 
	$self->{'mdc'} = $mdc;
	$self->SetSize($mdc->GetSize); 	
	$self->Refresh;
	$self->Update;
}


sub Render{
	my ($self) = @_;
	$mdc = Wx::MemoryDC->new();
	$self->{'mdc'} = $mdc;
	$mdc->SelectObject(Wx::Bitmap->new($self->GetSizeWH));
	$mdc->BeginDrawing() if $^O =~ /mswin/i;
	$mdc->Clear;
	my $data;
	my $row_length;
	my $complete_size;
	if(defined $self->{'bmp'}){	
		my $image = Wx::Image->new($self->{'bmp'});
		$data = $image->GetData();
		$row_length = $image->GetWidth * 3;
		$complete_size = $row_length * $image->GetHeight;
	}
	else{
		$data = $self->{'data'};
		$row_length = $self->{'size'}->[0] * 3;
		$complete_size = $row_length * $self->{'size'}->[1];
	}
	my ($y,$x) = (0,0);
	my $index = 0;
	my $row = 0;
    while(1){
		last if $index >= $complete_size;
		if($index != 0 && $index % $row_length == 0){
			$row++;	
		}
		my $bcolour = substr($data,0,3);
		$data = substr($data,3);
		my ($r,$g,$b) = unpack ('CCC',$bcolour);
		$index += 3;
		if(defined $self->{'trans_colour'}){
			if( $r == $self->{'trans_colour'}->[0] &&
				$g == $self->{'trans_colour'}->[1] && 
				$b == $self->{'trans_colour'}->[2]){
		
					next;
			
			}
		}
		else{
			if (($r == 165 && $g == 134 && $b == 179)){ # transparent pixel
				next;
			}
		}
		my @point = (($index % $row_length) / 3,$row);
		$self->{'pen'}->SetColour($r,$g,$b);
		$mdc->SetPen($self->{'pen'});
		$mdc->DrawPoint(@point);
	}
	$mdc->EndDrawing() if $^O =~ /mswin/i;
}


sub OnPaint{
	my ($self,$evt) = @_;
	defined $self->{'bmp'} or defined $self->{'mdc'} or defined $self->{'data'} or return undef; 
	my $pdc = Wx::PaintDC->new($self);
	unless($self->{'mdc'}){
		$self->Render;
	}
	$pdc->Blit(0,0,$self->GetSizeWH,$self->{'mdc'},0,0);
	$evt->Skip;
}

sub Destroy{
	my ($self) = @_;
	delete $self->{'brush'};
	delete $self->{'pen'};
	delete $self->{'data'};
	delete $self->{'mdc'};
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;