#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Registry.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: 


package SAPDB::Install::Registry;

require SDB::Common::Exporter;
our @ISA = qw (SDB::Common::Exporter);

our @EXPORT = qw (
	getmax getInstalledReleases readRuntimes removeKey readIndepPath
	readValues readSubKeys createUninstallEntry removeUninstallEntry
	setRenamedFiles readini writeini setSysPath createKey getInstances
	deleteSAPDBTech removeFromSysPath readGlobals AddDBEvtHandler
	GetSQLStudioUninstallString GetDBMGuiUninstallString);

use SAPDB::Install::StdIO;
use SAPDB::Install::System;
use SAPDB::Install::SetUser;
use SAPDB::Install::GetParam;
use SAPDB::Install::Values;
use SAPDB::Install::Tools;
use SAPDB::Install::Trace;
use SAPDB::Install::System::Win32::Registry;

if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
	require SAPDB::Install::Portable::Registry;
	import SAPDB::Install::Portable::Registry;
}


sub readini{
	# sub return such a hash : $hash{$SECTION}{$KEY}=$VALUE;
	# $SECTION - in ini file something like this: [my section]
	# $KEY - in ini file  something like this: my key = xxxxx
	# $VALUE - in ini file something like this: xxxx = my value
	my $file=$_[0]; # full path to ini file
	my %returnvalue;
	unless(-f $file){
		$SAPDB::Install::Values::log->SetMsg("readini(): file \"$file\" not found\n")
			if defined $SAPDB::Install::Values::log;
		return undef;
	}
	unless (open(INI,$file)){
		$SAPDB::Install::Values::log->SetMsg("readini(): cannot open file \"$file\"\n")
			if defined $SAPDB::Install::Values::log;
		return undef;
	}
	my $section;
	while(my $line=<INI>){
		chomp($line);
		$_=$line;
		/^[\ ]*\[(\S.*\S)\][\ ]*$/ and $section=$1;
		/[\ ]*(\S.*\S)[\ ]*=[\ ]*(.*)/ and $returnvalue{$section}{$1}=$2;
	}
	close(INI);
	unless(%returnvalue){
		return undef;
	}
	return %returnvalue;
}

sub writeini{
	my ($file,$args)=@_;
	my %data = (); 
	my $exists = 0;
	my @statbuf;
	if(-f $file){
		$exists = 1;
		@statbuf = stat($file);
		%data = readini($file);
	}
	
	if($args->{'add_key'}){
		foreach my $section (keys(%{$args->{'add_key'}})){
			my %values = ();
			if(exists $data{$section} and ref($data{$section}) eq 'HASH'){
				%values = %{$data{$section}};
			}
			foreach my $key (keys(%{$args->{'add_key'}->{$section}})){
				$values{$key} = $args->{'add_key'}->{$section}->{$key};
			}
			$data{$section} = \%values;
		}
	}

	if($args->{'delete_key'}){
		foreach my $section (keys(%{$args->{'delete_key'}})){
			my %values = ();
			if(exists $data{$section} and ref($data{$section}) eq 'HASH'){
				%values = %{$data{$section}};
			}
			foreach my $key (keys(%{$args->{'delete_key'}->{$section}})){
				delete $values{$key};
			}
			$data{$section} = \%values;
		}
	}

	unless(open(FD,'>'.$file.($exists ? '.sdbtmp' : ''))){
	 $SAPDB::Install::Values::log->SetMsg("ERR: writeini(): cannot open/create file \"$file".($exists ? '.sdbtmp' : '')."\"\n")
		if defined $SAPDB::Install::Values::log;
	 return 0;
	}
	foreach my $section (keys(%data)){
		print FD "[$section]\n";
		my %inner = %{$data{$section}};
		foreach my $valuename (keys(%inner)){
			print FD "$valuename=$inner{$valuename}\n";
		}
	}
	close (FD);
	if($exists){
		unlink($file);
		rename("$file.sdbtmp",$file);
		unless($^O =~ /mswin/i){
			chmod($statbuf[2],$file) or
				print2stderr("writeini(): cannot mode of $file\n");
			chown($statbuf[4],$statbuf[5],$file) or
				print2stderr("writeini(): cannot change owner of $file\n");
		}
	}
	return 1;
}


sub readValues{
	#sub retrun hash of values
	my $root=$_[0];
	unless($root){
		 $root = &HKEY_LOCAL_MACHINE;
	}
	my $keyname=$_[1];
	my $notexpand=$_[2];# true -> dont expand expandstring 
	unless(RegOpenKeyEx($root, $keyname, 0, &KEY_READ , $key ) == 0){ 
		$SAPDB::Install::Values::log->SetMsg("Can't open registry key \"HKEY_LOCAL_MACHINE\\$keyname\": $!\n")
			if defined $SAPDB::Install::Values::log;
		return undef;
	}
	my $i=0;
	my %returnvalue;
	RegQueryInfoKey ($key, 0, 0, 0, 0, 0, 0, $ValCount, 0, 0, 0, 0);
	for ($index = 0; $index < $ValCount; $index++){
		RegEnumValue( $key, $index ,$sValName, 0, 0, $piType, $pValData, 0 );
		if(Win32::GetLastError()==259){last;}
		if($piType==&REG_BINARY){
			$pValData=unpack("H*",$pValData);	
		}
		elsif($piType==&REG_EXPAND_SZ and not $notexpand){
			ExpandEnvironmentStrings($pValData,$pValData,0);
		}
		$sValName = '(Default)' unless $sValName =~ /\S/;
		$returnvalue{$sValName}=$pValData;
	}
	unless(RegCloseKey( $key ) == 0){
		$SAPDB::Install::Values::log->SetMsg("Can't close registry key \"$root\\$keyname\": $!\n")
			if defined $SAPDB::Install::Values::log;
	}
	return %returnvalue;
}


sub removeIndepPaths{

	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		return 1;
	}

	unless (RegOpenKeyEx(&HKEY_LOCAL_MACHINE, 'SOFTWARE\SAP\SAP DBTech', 0, &KEY_ALL_ACCESS , $key ) == 0){
		$SAPDB::Install::Values::log->SetMsg("WRN: Can't open \"HKEY_LOCAL_MACHINE\\Software\\SAP\\SAP DBTech\" : $!\n")
			if defined $SAPDB::Install::Values::log;
		return 0;
	}
	unless(RegDeleteValue($key,'IndepData')==0){
		$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete Value \"HKEY_LOCAL_MACHINE\\Software\\SAP\\SAP DBTech\\IndepData\" : $!\n")
			if defined $SAPDB::Install::Values::log;
		return 0;
	}
	unless(RegDeleteValue($key,'IndepPrograms')==0){
		$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete Value \"HKEY_LOCAL_MACHINE\\Software\\SAP\\SAP DBTech\\IndepPrograms\" : $!\n")
			if defined $SAPDB::Install::Values::log;
		return 0;
	}
	$SAPDB::Install::Values::log->SetMsg("MSG: removed IndepPaths successfully from windows registry\n")
		if defined $SAPDB::Install::Values::log;
	
	unless(RegCloseKey( $key ) == 0){
		$SAPDB::Install::Values::log->SetMsg("Can't close registry key \"HKEY_LOCAL_MACHINE\\SOFTWARE\\SAP\\SAP DBTech\": $!\n")
			if defined $SAPDB::Install::Values::log;
	}
	return 1;

}


sub deleteSAPDBTechWin{
	removeIndepPaths();
	unless(RegOpenKeyEx(&HKEY_LOCAL_MACHINE, 'Software\SAP', 0, &KEY_ALL_ACCESS , $key ) == 0){
		$SAPDB::Install::Values::log->SetMsg("WRN: Can't open \"HKEY_LOCAL_MACHINE\\Software\\SAP\" : $!\n")
			if defined $SAPDB::Install::Values::log;
		return 0;
	}
	unless(RegDeleteKey($key,'SAP DBTech')==0){
		$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete \"HKEY_LOCAL_MACHINE\\Software\\SAP\\SAP DBTech\" : $!\n")
			if defined $SAPDB::Install::Values::log;
		return 0;
	}
	$SAPDB::Install::Values::log->SetMsg("MSG: cleaned windows registry successfully\n")
		if defined $SAPDB::Install::Values::log;
	return 1;
}

sub deleteSAPDBTechUX{
	$SAPDB::Install::Values::owner = undef;
	$SAPDB::Install::Values::group = undef;
	if(-f $SAPDB::Install::Values::ini_path){
		unless(unlink($SAPDB::Install::Values::ini_path)){
			$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete \"$SAPDB::Install::Values::ini_path\"\n")
				if defined $SAPDB::Install::Values::log;
		}
	}
	else{
		$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete \"$SAPDB::Install::Values::ini_path\" - file dont exist\n")
			if defined $SAPDB::Install::Values::log;
	}
	
	if(-f $SAPDB::Install::Values::ini_path.'_lock'){
		unlink($SAPDB::Install::Values::ini_path.'_lock');
	}
	
	my $extra_ini  = $SAPDB::Install::Values::ini_path;
	
	$extra_ini =~ s/^\/usr\//\/var\//;
	
	if(-f $extra_ini){
		unlink($extra_ini);
	}
	
	$extra_ini .= '_lock';
	
	if(-f $extra_ini){
		unlink($extra_ini);
	}
	
	if(-f $SAPDB::Install::Values::glob_path){

		my $globals = readGlobals();
 		if (exists $globals->{'SdbOwner'}){
 			my $uid = getpwnam($globals->{'SdbOwner'});
 			if (defined $uid and $uid){
 				deleteUser($globals->{'SdbOwner'});				
 			}
 			my $gid = getgrnam($globals->{'SdbGroup'});
 			if (defined $gid and $gid){
 				deleteGroup($globals->{'SdbGroup'});				
 			}
 		}

		unless(unlink($SAPDB::Install::Values::glob_path)){
			$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete \"$SAPDB::Install::Values::glob_path\"\n")
				if defined $SAPDB::Install::Values::log;
		}
		my ($path) = ($SAPDB::Install::Values::glob_path =~ /(^.+)\/[^\/]+$/);
		if (-d $path){
			opendir(DH,$path);
			my @entries = grep {not /^\.{1,2}$/} readdir(DH);
			closedir(DH);
			if($#entries == -1){
				rmdir($path);
			}
		}
	}
	else{
		$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete \"$SAPDB::Install::Values::glob_path\" - file dont exist\n")
			if defined $SAPDB::Install::Values::log;
	}
	
	$SAPDB::Install::Values::log->SetMsg("MSG: removed SAP DB Registry successfully\n")
		if defined $SAPDB::Install::Values::log;
	return 1;
}

sub deleteSAPDBTech{
	$SAPDB::Install::Values::indep_data_path = undef;
	$SAPDB::Install::Values::indep_prog_path = undef;
	
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		return deleteSAPDBTechPortableRoot($config);;
	}
	elsif($^O =~ /^mswin/i){
		return deleteSAPDBTechWin();
	}
	else{
		return deleteSAPDBTechUX();
	}
} 



sub setSysPath{
	# sub set system PATH in windows registry
	my $appendString = $_[0]; # string to extend system path
	
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		return 1;
	}
	
	unless($^O=~/.*mswin.*/i){
		return 1;
	}
	my $keyname="SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment";
	my %values = readValues(&HKEY_LOCAL_MACHINE,$keyname,1);
	$appendString=~s/\//\\/g; # replace "/" with "\"
	my $testString=$appendString;
	$testString=~s/\\/\\\\/g; # replace "\" with "\\" for pattern matching
	my $path;
	foreach my $keyname (keys(%values)){
		$keyname=~/^path$/i and $path=$keyname;
	}
	unless($path){
		print2stderr("setSysPath(): real name of \"path\" not found\n");
		return 0;
	}
	if($values{$path}=~/$testString/i){
		$SAPDB::Install::Values::log->SetMsg("MSG: SYSTEM PATH is up to date\n")
			if defined $SAPDB::Install::Values::log;
		return 2;
	}
	$appendString="$appendString;$values{$path}";
	(RegOpenKeyEx(&HKEY_LOCAL_MACHINE, $keyname, 0, &KEY_SET_VALUE, $key ) == 0) 
	or  print2stderr("setSysPath(): cannot open HKEY_LOCAL_MACHINE\\$keyname: $!\n") and return 0;
	(RegSetValueEx($key,$path,0,&REG_EXPAND_SZ,$appendString,0)==0) or print2stderr("setSysPath(): cannot set value\n") and return 0;
	print2stdout("system path updated successfully\n");
	print2stdout("please reboot your system after installation to take effect changes\n");
	(RegCloseKey($key)==0) or print2stderr("setSysPath(): cannot close HKEY_LOCAL_MACHINE\\$keyname\n");
	$SAPDB::Install::Config{'SysPathChanged'} = 1;
	return 1;
}


sub removeFromSysPath{
	my ($removeString) = @_; # string to remove from system path
	
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		return 1;
	}
	
	$^O=~/mswin/i or return 1;
	my $keyname="SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment";
	my %values = readValues(&HKEY_LOCAL_MACHINE,$keyname,1);
	$removeString=~s/\//\\/g; # replace "/" with "\"
	my $testString=$removeString;
	$testString=~s/\\/\\\\/g; # replace "\" with "\\" for pattern matching
	my $path;
	foreach my $keyname (keys(%values)){
		$keyname=~/^path$/i and $path=$keyname;
	}
	unless($path){
		print2stderr("removeFromSysPath(): real name of \"path\" not found\n");
		return 0;
	}
	if($values{$path}=~/$testString/i){
		$SAPDB::Install::Values::log->SetMsg("MSG: SAP DB paths found in PATH\n")
			if defined $SAPDB::Install::Values::log;
	}
	else{
		$SAPDB::Install::Values::log->SetMsg("MSG: SAP DB paths not found in PATH\n")
			if defined $SAPDB::Install::Values::log;
		return 1;
	}
	my $newValue = $values{$path};
	$newValue =~ s/$testString;*//i;
	(RegOpenKeyEx(&HKEY_LOCAL_MACHINE, $keyname, 0, &KEY_SET_VALUE, $key ) == 0) 
	or  print2stderr("removeFromSysPath(): cannot open HKEY_LOCAL_MACHINE\\$keyname: $!\n") and return 0;
	(RegSetValueEx($key,$path,0,&REG_EXPAND_SZ,$newValue,0)==0) or print2stderr("removeFromSysPath(): cannot set value\n") and return 0;
	print2stdout("system path cleaned successfully\n");
	print2stdout("please reboot your system after uninstallation to take effect changes\n");
	(RegCloseKey($key)==0) or print2stderr("removeFromSysPath(): cannot close HKEY_LOCAL_MACHINE\\$keyname\n");
	return 1;
	
}


sub readSubKeys{
	#return array of sub keys
	my $root=$_[0];
	my $keyname=$_[1];
	unless($root){
		$root=&HKEY_LOCAL_MACHINE;
	}
	unless(RegOpenKeyEx($root, $keyname, 0, &KEY_READ , $key ) == 0){ 
		$SAPDB::Install::Values::log->SetMsg("Can't open registry key \"HKEY_LOCAL_MACHINE\\$keyname\": $!\n")
			if defined $SAPDB::Install::Values::log;
		return undef;
	} 
	my $i=0;
	my @returnvalue;
	RegQueryInfoKey ($key, 0, 0, 0, my $KeyCount, 0, 0, 0, 0, 0, 0, 0);
	for (my $index = 0; $index < $KeyCount; $index++){
		RegEnumKeyEx($key , $index, $sName, 0, 0, 0, 0,0 );
		if(Win32::GetLastError()==259){last;}
		push(@returnvalue,$sName);		
		
	}
	unless(RegCloseKey( $key ) == 0){
		$SAPDB::Install::Values::log->SetMsg("Can't close registry key \"$root\\$keyname\": $!\n")
			if defined $SAPDB::Install::Values::log;
	}
	return @returnvalue;
}



sub createKey{
	# sub create key in registry recursively
	my $hkey = $_[0]; # HKEY_* 
	my $key = $_[1];  # keyname
	unless($hkey){
		$hkey=&HKEY_LOCAL_MACHINE;
	}
	$key=~ /^[^\\]*$/ and return 1; # cant create first level key
	my $tmpkey=$key;
	$tmpkey=~s/^(.*)\\[^\\]{1,}$/$1/;
	createKey($hkey,$tmpkey) or return 0;
	(RegCreateKeyEx($hkey,$key,0,0,&REG_OPTION_NON_VOLATILE ,&KEY_CREATE_SUB_KEY  ,0,$newkey,$piDisp)==0) or print2stderr("createKey(): cannot create subkey \"$key\"\n") and return 0;
	unless(RegCloseKey($newkey)==0){
		$SAPDB::Install::Values::log->SetMsg("WRN: createKey(): cannot close new subkey \"$key\"\n")
			if defined $SAPDB::Install::Values::log;
	}
	if($piDisp==&REG_CREATED_NEW_KEY){
		$SAPDB::Install::Values::log->SetMsg("MSG: created registry subkey \"$key\" successfully\n")
			if defined $SAPDB::Install::Values::log;
	}
	elsif($piDisp==&REG_OPENED_EXISTING_KEY){
		$SAPDB::Install::Values::log->SetMsg("MSG: registry subkey \"$key\" exist\n")
			if defined $SAPDB::Install::Values::log;
	}
	return 1;
}


sub removeKey{
	my ($hRoot,$keyname,$mode)=@_;
	
	local *root = $hRoot;	
	
	#
	# modes:	0 -> remove only if key is empty
	#			1 -> remove it with all empty subkeys
	#			2 -> remove it with all subkeys and values
	#		  
	
	unless(exists $root{'handle'} and defined $root{'handle'}){
		eval{
			$root{'handle'} = &{$root{name}};  
		};
		if($@){
			$SAPDB::Install::Values::log->SetMsg("unknown registry root \"{$root{name}\": $@\n")
				if defined $SAPDB::Install::Values::log;
			return 0;
		}
	}

	if($mode == 2){
		my %values = readValues($root{'handle'},$keyname);
		my $key;
		unless(RegOpenKeyEx($root{'handle'},$keyname, 0, &KEY_ALL_ACCESS , $key ) == 0){
			$SAPDB::Install::Values::log->SetMsg("WRN: Can't open \"$root{'name'}\\$keyname\" : $!\n")
				if defined $SAPDB::Install::Values::log;
			return 0;
		}
		foreach my $value_name (keys(%values)){
			unless(RegDeleteValue($key,$value_name) == 0){
				$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete Value \"$root{'name'}\\$keyname\\$value_name\" : $!\n")
					if defined $SAPDB::Install::Values::log;
				return 0;
			}
		}
		unless(RegCloseKey( $key ) == 0){
			$SAPDB::Install::Values::log->SetMsg("Can't close registry key \"$root{'name'}\\$keyname\": $!\n")
				if defined $SAPDB::Install::Values::log;
		}
	}
	if($mode >= 1){
		my @subkeys = readSubKeys($root{'handle'},$keyname);
		foreach my $subkey (@subkeys){
			removeKey($hRoot,"$keyname\\$subkey",$mode) or return 0;
		} 
	}
	if($mode >= 0){
		my ($parent_keyname,$subkeyname) = ($keyname =~ /^(.*)\\([^\\]+)\\{0,1}$/);
		my $key;
		
		unless(RegOpenKeyEx($root{'handle'},$parent_keyname, 0, &KEY_ALL_ACCESS , $key ) == 0){
			$SAPDB::Install::Values::log->SetMsg("WRN: Can't open \"$root{'name'}\\$parent_keyname\" : $!\n")
				if defined $SAPDB::Install::Values::log;
			return 0;
		}
		
		unless(RegDeleteKey($key,$subkeyname)==0){
			$SAPDB::Install::Values::log->SetMsg("WRN: cannot delete \"$root{'name'}\\$parent_keyname\\$subkeyname\" : $!\n")
				if defined $SAPDB::Install::Values::log;
			return 0;
		}
		
		unless(RegCloseKey( $key ) == 0){
			$SAPDB::Install::Values::log->SetMsg("Can't close registry key \"$root{'name'}\\$parent_keyname\": $!\n")
				if defined $SAPDB::Install::Values::log;
		}
	}
	return 1;
}




#sub return max from arg list
sub getmax{
    local %tmp=@_;
    $max=0;
    my $rel;
	foreach  $rel (%tmp){
		if($rel > $max){
		    $max=$rel;
		}
    }
    return $max;
}


#sub reads releases and dbroots from ini file
sub getInstalledReleasesUX{
    my %returnvalue;
    my $indep_data = (readIndepPath())[0];
	foreach my $path ($indep_data.'/config/Installations.ini',$SAPDB::Install::Values::ini_path){
		if(-f $path){
			my %hash = readini($path);
			return undef unless defined %hash;
			foreach $inst (keys( %{$hash{'Installations'}})){
				next if exists $returnvalue{$inst};
				$_=$hash{'Installations'}{$inst};	
				/([0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*),/ and $returnvalue{$inst}=$1;
			}
		}	
	}
	
	foreach $dbroot (keys %returnvalue){
			my $release=getRelease("$dbroot/pgm/kernel");
			if($release){
				$returnvalue{$dbroot}=$release;
			}
	}
	return %returnvalue;
}

sub getInstancesUX{
	my %returnvalue;
	my $indep_data = (readIndepPath())[0];
	foreach my $path ($indep_data.'/config/Databases.ini',$SAPDB::Install::Values::ini_path){
		if(-f $path){
			my %hash=readini($path);
			my $tmp=$hash{'Databases'};
			my %dbroot=%$tmp;
			foreach my $db (keys(%dbroot)){
				#filter service databases ( '_' for < 7.4 )
				unless($db=~/^_/ || $db=~/^\./){
						$returnvalue{$db}= $dbroot{$db} unless exists $returnvalue{$db};
				}
			}
		}	
	}
	return %returnvalue;
}


sub getInstancesWin{
	#HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\SAP DBTech-TST
	my @subkeys = readSubKeys(0,'SYSTEM\CurrentControlSet\Services');
	my %returnvalue;
	my @err_instances;
	foreach my $subkey (@subkeys){
		my $key=$subkey;
		if($subkey=~s/^SAP\ DBTech-//){
			
			#filter service databases ( '_' for < 7.4 )
			next if ($subkey=~/^_/ || $subkey=~/^\./);
			my %db;	
			if($subkey =~ /\s+\(\S+\)$/){
				($db{'name'},$db{'knltype'}) = ( $subkey =~ /^(\S+)\s+\((\S+)\)/);
			}
			else{
				$db{'name'} = $subkey;
				$db{'knltype'} = 'fast';
			} 
			
			my %values = readValues(0,"SYSTEM\\CurrentControlSet\\Services\\$key");	
			$db{'image'}=$values{'ImagePath'};
			$db{'path'} = $db{'image'};
			
			$db{'path'} =~ s/\\pgm\\[^\\]+\.exe//;
			
			TraceMsg("found instance \"$db{name}\" in \"$db{path}\" ($db{knltype}) \n",3,\$DEBUG);
			
			if(exists $returnvalue{$db{'name'}}){
				if(normalizePath($returnvalue{$db{'name'}}) ne normalizePath($db{'path'})){
					$db{'path'} =~ s/\\/\//g;
					print2stderr("ERR: found multiple installation pathes of instance $db{name}\n".
								 "ERR: ".$returnvalue{$db{'name'}}." vs. $db{path} [kernel type $db{knltype}]\n".
								 "ERR: ignore instance \"$db{name}\"\n");
					push @err_instances, $db{'name'};				
				}
			}
			else{
				$db{'path'} =~ s/\\/\//g;	
				$returnvalue{$db{'name'}} = $db{'path'}; 
			}
		}
	}
	
	foreach my $db (@err_instances){
		if(exists $returnvalue{$db}){
			delete $returnvalue{$db};
		}
	}
	
	return %returnvalue;
}


*getInstances =
	exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'} ? \&getInstancesPortableRoot : 
	$^O =~ /^MsWin/i ? \&getInstancesWin :
	\&getInstancesUX;


sub readRuntimesUX{
	my %returnvalue;
	my $indep_data = (readIndepPath())[0];
	my $ext = 'so';
	if ($^O =~ /hpux/i){
		require SAPDB::Install::SysInfo;
		my $sysinfo = SAPDB::Install::SysInfo::GetSystemInfo ();
		unless (exists $sysinfo->{'architecture'} and 
		    $sysinfo->{'architecture'} =~ /ia64/i){
			$ext = 'sl';
		}
	}
	foreach my $path ($indep_data.'/config/Runtimes.ini',$SAPDB::Install::Values::ini_path){
		next unless -f $path;
		my %hash = readini($path) or print2stderr("cannot read $path\n") and return;
		my %releases = %{$hash{'Runtime'}};
		foreach my $rpath (keys(%releases)){
			next if exists $returnvalue{$rpath};
			$returnvalue{$rpath}=$releases{$rpath};
			$returnvalue{$rpath}=~s/,.*$//;
			next unless($returnvalue{$rpath} =~ /\.0+$/);
			foreach my $subpath ('lib','lib/lib64'){
				my $filename="$rpath/$subpath/libpcr.$ext";
				-f $filename or next;
				my ($maj,$min,$cor,$patch)=SAPDB::Install::BuildInfo::GetBuild($filename);
				my $release = "$maj.$min.$cor.$patch";
				$release =~ /\d/ and $returnvalue{$rpath}=$release and last;
			}	
		}
	}
	return %returnvalue;
}


sub readRuntimesWin{
	my @paths=readSubKeys(0,"SOFTWARE\\SAP\\SAP DBTech\\Runtime");
	my %returnhash;
	foreach my $path (@paths){
		my $origpath=$path;
		$path =~ s/\\/\//g;
		my %values = readValues(0,"SOFTWARE\\SAP\\SAP DBTech\\Runtime\\$origpath");
		my $release=$values{''};
		$release=~s/,.*$//;
		$returnhash{$origpath}=$release;
		next if($release =~ /\d/ and $release !~ /\.0+$/);
		my $filename = "$path/pgm/libpcr.dll";
		-f $filename or $filename = "$path/pgm/pgm64/libpcr.dll";
		-f $filename or next;
		my ($maj,$min,$cor,$patch)=SAPDB::Install::BuildInfo::GetBuild($filename);
		$release = "$maj.$min.$cor.$patch";
		$release  =~ /\d/ and $returnhash{$origpath}=$release;
	}
	return %returnhash;
}


*readRuntimes = 
	exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'} ? \&readRuntimesPortableRoot : 
	$^O =~ /^MsWin/i ? \&readRuntimesWin :
	\&readRuntimesUX;



sub getInstalledReleasesWin{
	my @inst = readSubKeys(0,"SOFTWARE\\SAP\\SAP DBTech\\Installations");
	($#inst==-1) and return undef;
	my %rel;
	foreach my $inst (@inst){
		my %hash=readValues(0,"SOFTWARE\\SAP\\SAP DBTech\\Installations\\$inst");
		$_=$hash{'VersionID'};
		/^(..)(..)(..)(..)$/ or print2stderr("getInstalledReleasesWin(): wrong version format\n");
		my $major=hex($1);
		my $minor=hex($2);
		my $corr=hex($3);
		my $patch=hex($4); 
		$inst =~ s/\\/\//g;
		$rel{$inst}="$major.$minor.$corr.$patch";
	}
	foreach my $dbroot (keys %rel){
		my $release=getRelease("$dbroot/pgm/kernel.exe");
		if($release){
			$rel{$dbroot}=$release;
		}
    }
	return %rel;
}

*getInstalledReleases = 
	# return all installed serverreleases in a hash -> key = instroot 
	exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'} ? \&getInstalledReleasesPortableRoot :
	$^O =~ /^MsWin/i ? \&getInstalledReleasesWin :
	\&getInstalledReleasesUX;


sub readGlobals{
	my $glob_path;
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		$glob_path = $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}.
			$SAPDB::Install::Values::glob_path;
	}
	else{
		$glob_path = $SAPDB::Install::Values::glob_path;
	}
	
	-f $glob_path or return undef;
	
	my %globs = readini($glob_path)  or
						print2stderr("can\'t read globals: $!\n");
	
	unless (exists $globs{'Globals'}){
		return undef;
	}
	
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
			$globs{'Globals'}->{'IndepData'} = $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}.'/'.$globs{'Globals'}->{'IndepData'};
			$globs{'Globals'}->{'IndepPrograms'} = (exists $SAPDB::Install::Config{'SDB_PORTABLE_PROGRAMS_ROOT'} ? 
						$SAPDB::Install::Config{'SDB_PORTABLE_PROGRAMS_ROOT'} :	
						$SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}).'/'.$globs{'Globals'}->{'IndepPrograms'};
	}		
	
	if (exists $globs{'Globals'}->{'SdbOwner'}){
		$SAPDB::Install::Values::owner = $globs{'Globals'}->{'SdbOwner'};
	}
	
	if (exists $globs{'Globals'}->{'SdbGroup'}){
		$SAPDB::Install::Values::group = $globs{'Globals'}->{'SdbGroup'};
	}
	
	return $globs{'Globals'};
}



#sub return paths of independent data and programs (list of 2 elements)
sub readIndepPathUX{
	if(-f $SAPDB::Install::Values::glob_path){
		my $globs = readGlobals();
		return $globs->{'IndepData'},$globs->{'IndepPrograms'};
	}
	else{
		my $ini = $SAPDB::Install::Values::ini_path;
		unless(-f $ini){
			$ini =~ s/^\/usr\//\/var\//;
			return () unless -f $ini;				
		}
		my %hash = readini($ini) or 
						print2stderr("cannot read SAP_DBTech.ini: $!\n");
		return $hash{'Globals'}{'IndepData'}, $hash{'Globals'}{'IndepPrograms'};
	}
}

sub readIndepPathWin{
    my $data_path;
    my $prog_path;
    my %hash=readValues(0,"SOFTWARE\\SAP\\SAP DBTech");
	$hash{'IndepData'}=~s/\\/\//g;
	$hash{'IndepPrograms'}=~s/\\/\//g;
	return $hash{'IndepData'}, $hash{'IndepPrograms'};
}


sub createUninstallEntry{
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		return 1;
	}
	$^O !~ /mswin/i && return 1;
	my $rc = 1;
	my ($name,$command) = @_;
	my $keyname = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\SAP DB (ALL)';
	createKey(0,$keyname);
	my $key;
	(RegOpenKeyEx(&HKEY_LOCAL_MACHINE, $keyname, 0, &KEY_SET_VALUE, $key ) == 0) 
	or  print2stderr("createUninstallEntry(): cannot open HKEY_LOCAL_MACHINE\\$keyname: $!\n") and return 0;
	(RegSetValueEx($key,'DisplayName',0,&REG_EXPAND_SZ,$name,0)==0) or print2stderr("createUninstallEntry(): cannot set value \"DisplayName\"\n") and $rc=0;
	(RegSetValueEx($key,'UninstallString',0,&REG_EXPAND_SZ,$command,0)==0) or print2stderr("createUninstallEntry(): cannot set value \"UninstallString\"\n") and $rc=0;
	unless(RegCloseKey( $key ) == 0){
		$SAPDB::Install::Values::log->SetMsg("Can't close registry key \"HKEY_LOCAL_MACHINE\\$keyname\": $!\n")
			if defined $SAPDB::Install::Values::log;
		$rc=0;
	}
	return $rc;	
}

sub removeUninstallEntry{
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		return 1;
	}
	$^O !~ /mswin/i && return 1;
	my $keyname = 'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\SAP DB (ALL)';
	return removeKey({'name' => 'HKEY_LOCAL_MACHINE', 'handle' => &HKEY_LOCAL_MACHINE},$keyname,2);
}



sub setRenamedFiles{
	$^O !~ /mswin/i && return 0;
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		return 1;
	}
	my @files = @_;
	my @tmp;
	foreach my $file (@files){
		$file =~ s/\//\\/g;
		push @tmp,'\??\\'.$file;
		push @tmp,''; 
	}
	@files = @tmp;
	my $keyname = 'SYSTEM\CurrentControlSet\Control\Session Manager';
	my $valuename = 'PendingFileRenameOperations';	
	my $key;
	my %values = readValues(&HKEY_LOCAL_MACHINE, $keyname);
	if(exists $values{$valuename} and defined $values{$valuename}){
		my @existingfiles = @{$values{$valuename}};
		push @files,@existingfiles;
	}
	(RegOpenKeyEx(&HKEY_LOCAL_MACHINE, $keyname, 0, &KEY_SET_VALUE, $key ) == 0) 
		or  print2stderr("setRenamedFiles(): cannot open HKEY_LOCAL_MACHINE\\$keyname: $!\n")
		and return 0; 	
	(RegSetValueEx($key,$valuename,0,&REG_MULTI_SZ,\@files,0) == 0)
		or print2stderr("setRenamedFiles(): cannot set value \"$valuename\"\n")
		and return 0;
	unless(RegCloseKey( $key ) == 0){
		$SAPDB::Install::Values::log->SetMsg("Can't close registry key \"HKEY_LOCAL_MACHINE\\$keyname\": $!\n")
			if defined $SAPDB::Install::Values::log;
		return 0;
	}
	return 1;
}


sub readIndepPath{
	my ($no_cache) = @_;
	if(	not $no_cache and $SAPDB::Install::Values::indep_prog_path =~ /\S/ and 
		$SAPDB::Install::Values::indep_data_path =~ /\S/){
		return ($SAPDB::Install::Values::indep_data_path,$SAPDB::Install::Values::indep_prog_path);
	}
	my @result;
	if (exists $SAPDB::Install::Config{'SDB_PORTABLE_ROOT'}){
		@result = readIndepPathPortableRoot();
	}
	elsif ($^O =~ /mswin/i){
		@result = readIndepPathWin(); 
	}
	else{
		@result = readIndepPathUX();
	}
	unless($no_cache){
		($SAPDB::Install::Values::indep_data_path,$SAPDB::Install::Values::indep_prog_path) = @result;
	}
	return @result;
}

sub GetDBMGui{
	return undef if $^O !~ /mswin/i;
	#HKEY_CLASSES_ROOT/DBMGui3.Application/Clsid
	my %values = readValues(&HKEY_CLASSES_ROOT,'DBMGui3.Application\Clsid');
	unless(exists $values{'(Default)'}){
		return undef;
	}
	my $clsid = $values{'(Default)'};
	
	#HKEY_CLASSES_ROOT/CLSID/{<clsid>}/LocalServer32

	%values = readValues(&HKEY_CLASSES_ROOT,'CLSID\\'.$clsid.'\LocalServer32');
	
	unless(exists $values{'(Default)'}){
		return undef;
	}

	return $values{'(Default)'}
}

sub GetDBMGuiUninstallString{
	return undef if $^O !~ /mswin/i;
	#HKEY_CLASSES_ROOT/DBMGui3.Application/Clsid
	my %values = readValues(&HKEY_CLASSES_ROOT,'DBMGui3.Application\Clsid');
	unless(exists $values{'(Default)'}){
		return undef;
	}
	my $clsid = $values{'(Default)'};
	
	#HKEY_CLASSES_ROOT/CLSID/{<clsid>}/LocalServer32

	%values = readValues(&HKEY_LOCAL_MACHINE,'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\\'.$clsid);
	
	unless(exists $values{'UninstallString'}){
		return undef;
	}
	return $values{'UninstallString'};	
}


sub GetSQLStudioUninstallString{
	return undef if $^O !~ /mswin/i;
	#HKEY_CLASSES_ROOT/SQLStudio.SQLSTO/Clsid
	my %values = readValues(&HKEY_CLASSES_ROOT,'SQLStudio.SQLSTO\Clsid');
	unless(exists $values{'(Default)'}){
		return undef;
	}
	my $clsid = $values{'(Default)'};
	
	#HKEY_CLASSES_ROOT/CLSID/{<clsid>}/LocalServer32

	%values = readValues(&HKEY_LOCAL_MACHINE,'SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\\'.$clsid);
	
	unless(exists $values{'UninstallString'}){
		return undef;
	}
	return $values{'UninstallString'};	
}


sub GetSQLStudio{
	return undef if $^O !~ /mswin/i;
	#HKEY_CLASSES_ROOT/SQLStudio.SQLSTO/Clsid
	my %values = readValues(&HKEY_CLASSES_ROOT,'SQLStudio.SQLSTO\Clsid');
	unless(exists $values{'(Default)'}){
		return undef;
	}
	my $clsid = $values{'(Default)'};
	
	#HKEY_CLASSES_ROOT/CLSID/{<clsid>}/LocalServer32

	%values = readValues(&HKEY_CLASSES_ROOT,'CLSID\\'.$clsid.'\LocalServer32');
	
	unless(exists $values{'(Default)'}){
		return undef;
	}

	return $values{'(Default)'}	
}

sub AddDBEvtHandler{
	my ($dbname,$feature_type) = @_;

	my $features = {
	
		'AUTO_UPDATE_STATISTICS' => {
									'pattern' => '^\s*UPDATE_STATISTICS=',
									'cfgline' => 'UPDATE_STATISTICS=dbmcli -d $DBNAME$ -u $DBMUSERPW$ -n $SERVERNAME$ sql_updatestat' 
							   },
		'AUTO_EXTEND' => {
									'pattern' => '^\s*DB_ABOVE_LIMIT=dbmevthndl_extendDB',
									'cfgline' => 'DB_ABOVE_LIMIT=dbmevthndl_extendDB  -evt \"$EVTALL$\" -l 90 -d $DBNAME$ -u $DBMUSERPW$ -n $SERVERNAME$' 
						}
	};

	return undef unless exists $features->{$feature_type};

	my $cfgfile = 'dbmevtdisp.cfg';
	my %instances = getInstances();
	my @inbuffer;
	unless(exists $instances{$dbname}){
		return undef;
	}
	my $params = SAPDB::Install::GetParam::GetParam($dbname)->{'data'};
	my $rundir = $params->{'RUNDIRECTORY'};
	
	return undef unless -d $rundir;

	my $infile;
	my $outfile = "$rundir/$cfgfile";
	my $create_new_file = 0;

	if(-f $outfile){
		$infile = $outfile;
	}
	elsif(-f $instances{$dbname}.'/env/'.$cfgfile){
	
		#
		# configuration template
		#
		$create_new_file = 1;
		$infile = $instances{$dbname}.'/env/'.$cfgfile;
	}
	else{
		return undef;
	}
	
	open(IN,$infile);
	while(<IN>){
		chomp;
		push @inbuffer,$_;
	}
	close(IN);

	my $found = 0;

	foreach my $line (@inbuffer){
		if($line =~ /$features->{$feature_type}->{'pattern'}/){
			$found = 1;
			last;
		}
	}

	if($create_new_file){
		open(OUT,'>'.$outfile);
		foreach my $line (@inbuffer){
			print OUT $line."\n";
		}
	}
	elsif(not $found){
		open(OUT,'>>'.$outfile);
	}
	else{
		return 1;
	}

	unless($found){
		print OUT $features->{$feature_type}->{'cfgline'}."\n"; 
	}

	close(OUT);

	unless($^O =~ /mswin/i){
		my $globs = readGlobals();
		chmod(0660,$outfile);
		my $uid = getpwnam($globs->{'SDBOwner'});
		my $gid = getgrnam($globs->{'SDBGroup'});
		chown($uid,$gid,$outfile);	
	}

	return 1;	
}



1;
