#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Uninstall/RemoveFilesInUse.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: - Windows and HP-UX only
#		- all other supported plattforms are able to delete such files
#		- move busy binaries into a temp folder
#		- generate registry entry for deletion at next windows start 
#

package SAPDB::Install::Uninstall::RemoveFilesInUse;

require SDB::Common::Exporter;
our @ISA = qw (SDB::Common::Exporter);

use SAPDB::Install::System;
use SAPDB::Install::Registry;
use SAPDB::Install::StdIO;
use SAPDB::Install::Values;
use SAPDB::Install::SetDebug;
use SAPDB::Install::Tools;
use SAPDB::Install::Misc;
use SAPDB::Install::Trace;
use SAPDB::Install::System::Win32::API;

our @EXPORT = qw (killUsedFiles);

our $DEBUG=0;

sub setFileRenamedOperation{
	my (@files) = @_;
	my $rc = 1;
	my $useOwnRegistryAccess = 0;
	if($useOwnRegistryAccess){
		$rc = setRenamedFiles(@files);
	}
	else{
		foreach my $file (@files){
			$file =~ s/\//\\/g;
			my ($lrc,$errtxt) = SAPDB::Install::System::Win32::API::MoveFileAfterReboot($file);	
			unless($lrc == 0){
				print2stderr("cannot mark \"".$file."\" for deletion after reboot: $errtxt\n");
				$rc = 0;
			}
		}	
	}
	return $rc;
}


my %tmps;

sub getTmpDir{
	my ($drive) = @_;
	defined $tmps{$drive} and return 1;
	my $found = 0;
	foreach my $dir ('TEMP','TMP'){
		if( -d "$drive/$dir"){
			$tmps{$drive} = "$drive/$dir";
			$found = 1;
			last;
		}
	}
	unless($found){
		# no tmp directory for drive found, make own one
		my $tmpdir = "$drive/TEMP";
		mkdir($tmpdir,0777) or print2stderr("cannot create temporary directory \"$tmpdir\"\n") and return 0; 
		$tmps{$drive} = $tmpdir;
	}
	return 1; 
}


sub killUsedFiles{
	return 0 unless($^O =~ /mswin|hpux/i); # only for win && hpux
	my ($prog,@files) = @_;
	if($^O =~ /hpux/i){
		my $rmfile = "$prog/pgm/rmfile";
		if(-x $rmfile){
			foreach my $file (@files){
				my $rc = callsystem("$rmfile $file",undef,1);
				if ($rc != 0){
					$SAPDB::Install::Values::log->SetMsg("WRN:rmfile returned $rc\n")
						if defined $SAPDB::Install::Values::log;
				}
			}
		}
		else{
			$SAPDB::Install::Values::log->SetMsg("WRN: cannot remove busy file $file: program rmfile [$rmfile] not found\n")
				if defined $SAPDB::Install::Values::log;
		}
		return 0;
	}
	
	my @renamedFiles;
	
	foreach my $file (@files){
		$file =~ s/\\/\//g;
		-f $file or TraceMsg("file \"$file\" not found\n",4,\$DEBUG) and next;	
		my ($drive,$filename) = ( $file =~ /^(..).*\/([^\/]+)$/) ;
		$filename =~ /\S/ or next;
		$drive =~ tr/[A-Z]/[a-z]/;
		unless($drive =~ /^[a-z]:$/){
			print2stderr("drive unknown\n");
			next;
		}
		getTmpDir($drive) or print2stderr("cannot get tmp dir of filesystem\n") and next;	
		my $destination = "$tmps{$drive}/$filename.del";
		if(-f $destination){
			my $count = 0;
			while(-f "$destination$count"){$count++;}
			$destination = "$destination$count";
			TraceMsg("move to $destination\n",4,\$DEBUG);
		}
		rename($file,$destination) or print2stderr("cannot rename used file \"$file\": $!\n") and next; 	
		$destination =~ s/\//\\/g; 
		push @renamedFiles,$destination;
	}
	setFileRenamedOperation(@renamedFiles) or return 0;	
	return 1;
}

1;