#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/Install.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: Installation dialog - extract files and do register calls
#

package SAPDB::Install::Wizard::Dialog::Install;

use Wx qw (
	wxEXPAND wxTOP wxALIGN_CENTER wxALL
	wxLEFT wxVERTICAL wxSUNKEN_BORDER);

use base SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::Control::StaticText;
use SAPDB::Install::StdIO;
use SAPDB::Install::Misc; 
use SAPDB::Install::Window::ExecPlan;


sub new{
	my($this ) = shift->SUPER::new(@_);
	return $this;
}


sub init{
	my ($self,$config) = @_;
	if(	defined $config->{'dbupdate'} and 
		defined $config->{'dbupdate'}->{'registry_instancedata'} and
		$config->{'dbupdate'}->{'registry_instancedata'}->{'SOFTWARE_OK'}){

		return {'auto_next' => 1};
	}	
	$self->Enable(1);
	$self->Show(1);
	my $install = 0; 
	
	my $install_count = 0;

	my @steps;
	foreach my $packobj (@{$config->{'packages'}}){
		next if (${$packobj->Skip});
		$install_count++;
		push @steps,{'Label' => $packobj->DispName,'State' => 'PENDING'};
	}
	
	my $top_sizer = Wx::BoxSizer->new(wxVERTICAL);
	$self->{'plan'} = SAPDB::Install::Window::ExecPlan->new($self,-1);
	$top_sizer->Add($self->{'plan'},1,wxALL | wxEXPAND, 20);
	$self->SetSizer($top_sizer);
	$self->SetAutoLayout(1);
	$self->Layout;
	
	$self->{'plan'}->init(\@steps,{'with_progress' => 1});

	# after check phase its allowed to write into install registry file
	#if($install_count > 0){
		my $save_dirty = $config->{'container'}->InstReg->dirty;
		$config->{'container'}->InstReg->DenyWrite(0);
		$config->{'container'}->InstReg->dirty(0) if $save_dirty == 0;
	#}	
	$self->{'initialized'} = 1;
	$self->{'plan'}->Show(1);
	$self->{'plan'}->Refresh(1);
	$self->{'plan'}->Update;
	$self->GetWizard()->KillPrevSteps();
	return {'auto_next' => 1};
}

sub Process{
	my ($self,$config) = @_;
	
	if(	defined $config->{'dbupdate'} and 
		defined $config->{'dbupdate'}->{'registry_instancedata'} and
		$config->{'dbupdate'}->{'registry_instancedata'}->{'SOFTWARE_OK'}){

		return 1;
	}		
	
	$self->{'plan'}->Next();
	foreach my $packobj (@{$config->{'packages'}}){
		${$packobj->Skip} and next;
		
		if($packobj->{'dep_error'}){
			$config->{'container'}->Log->SetMsg('ERR: package '.$packobj->Name.': dependency error: package '.$packobj->{'dep_error'}." failed\n");
			$self->{'plan'}->Next('FINISHED_ERROR');
			next;
		}
		
		$packobj->FlushIntoReg(1);
		
		
		#$self->{'caption'}->setText('Installing MaxDB Package '.$packobj->DispName);
		
		if($^O =~ /mswin/i and $packobj->MappedFiles){
				my @mapped_files = (); 
				foreach my $file (@{$packobj->MappedFiles}){
					push @mapped_files, $packobj->MainPath->{'value'}.'/'.$file;
				}
				unless(killUsedFiles(@mapped_files)){
					$self->SetError("Error: Cannot Remove Mapped Files",
									"Unable To Move Mapped Files - \n". 
									"please terminate event viewer or other windows\n".
									"management console tools to avoid mapped files."
									);
					return undef;
					
				}
		}
		
		my $subpackobjs;

		$packobj->RegData->setValid(0);
		#$self->{'caption'}->Refresh();
		my $progress_value = 0;
		
		my @files = %{$packobj->FileList};
		if (defined $packobj->DbgFiles){
			push @files, (%{$packobj->DbgFiles});
		}
		my $scale = $#files + 1;
		
		if(defined $packobj->register){
			$scale++;
		}
		
		if(defined $packobj->postinstall){
			$scale++;
		}
		
		
		if($scale == 0){
			$self->{'plan'}->SetProgress(50);
		}
						
		my $eval_rc = eval{
	
		if($packobj->Update){
			$subpackobjs = update($packobj,{'callback_progress' =>
								sub {
									$progress_value++; 									
									my $value = int(100 * $progress_value / $scale);
									$self->{'plan'}->SetProgress($value);
									$self->CallbackRefresh;
									#$self->{'progresswin'}->SetProgress(@_)
								},
								'callback_initprogress' => 
								sub {
									#$self->{'progresswin'}->init(@_)
								}
							});
		}
		else{
			newinstall($packobj,{'callback_progress' =>
								sub {
									$progress_value++; 									
									my $value = int(100 * $progress_value / $scale);
									$self->{'plan'}->SetProgress($value);
									$self->CallbackRefresh
									#$self->{'progresswin'}->SetProgress(@_)
								},
								'callback_initprogress' => 
								sub {
									$self->CallbackRefresh
									#$self->{'progresswin'}->init(@_)
								} 
						});
		}
		1;
		};
		
		die($@) if($@ =~ /^SDB_closing/);
		
		if($@){
			$self->SetError('Installation of Package '.$packobj->DispName." failed",$@);
			$packobj->{'errstate'} = 1;
		}


		unless($eval_rc == 1){
			return $eval_rc;
		}
	
		if(	not $packobj->{'errstate'} and defined $packobj->postinstall){
			eval{
				&{$packobj->postinstall};
				$progress_value++; 									
				my $value = int(100 * $progress_value / $scale);
				$self->{'plan'}->SetProgress($value);
				$self->CallbackRefresh;
			}; 
			die($@) if($@ =~ /^SDB_closing/);
			if($@){
				$self->SetError('error in postinstall() of '.$packobj->DispName,$@);
				$packobj->{'errstate'} = 1;
				#return {'errstate' => 1};
			}
		}
		if(	not $packobj->{'errstate'} and defined $packobj->register){
			eval{
				&{$packobj->register};
				$progress_value++; 									
				my $value = int(100 * $progress_value / $scale);
				$self->{'plan'}->SetProgress($value);
				$self->CallbackRefresh;
			};
			die($@) if($@ =~ /^SDB_closing/);
			if($@){
				$self->SetError('error in register() of '.$packobj->DispName,$@);
				$packobj->{'errstate'} = 1;
				#return {'errstate' => 1};
			}
		}

		if($scale == 0){
			$self->{'plan'}->SetProgress(100);
		}

		$packobj->RegData->genFileInfos();
		if($packobj->{'errstate'}){
			$self->{'errstate'} = 1;
			my $tree_node = $config->{'collector'}->package_index->{$packobj->Name}->{'tree_ref'};
			my $list = $config->{'collector'}->GetPackageList($tree_node,1);
			foreach my $child_package_name (@$list){
				foreach my $packref (@{$config->{'packages'}}){
					if($packref->Name eq $child_package_name){
						$packref->{'dep_error'} = $packobj->Name;
					}
				}													
			}
			$self->{'plan'}->Next('FINISHED_ERROR');
		}		
		else{
			$packobj->RegData->setValid(1);
			if(defined $subpackobjs){
				local *subs = $subpackobjs;
				foreach my $subpack (@subs){
					if(defined $$subpack->register){
						$$subpack->Registry->Log->SetMsg("register subpackage ".$$subpack->Name."\n");
						eval{
							&{$$subpack->register};
						};
						die($@) if($@ =~ /^SDB_closing/);
						if($@){
							$self->SetError("error in register() of subpackage ".$$subpack->Name,$@);
							return {'errstate' => 1};
						}
					}
					$$subpack->setValid(1);	
				}	
			}
			if($packobj->Name eq 'Database Kernel'){
				if(defined  $config->{'dbupdate'}){
					$config->{'container'}->InstReg->setInstanceData($config->{'dbupdate'}->{'dbname'},{'SOFTWARE_OK' => 1});
				}
				$self->{'kernel_ok'} = 1;
			}
			$self->{'plan'}->Next('FINISHED_SUCCESS');
		}
	}
	
	if($config->{'comm_stopped_by_software_update'}){
		$config->{'comm'}->start;
	}
	
	
	if($self->{'kernel_ok'} and defined $config->{'container'}->Template){
		return 1;
	}
	
	if($self->{'errstate'}){
		return {'errstate' => 1};
	}

	return 1;
}

sub CallbackRefresh{
 my ($self) = @_;
	&{$SAPDB::Install::Values::RefreshGui}();
	return 1;
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'plan'}){ 		
		$self->{'plan'}->Destroy;
		delete $self->{'plan'};
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}



1;
