#!/usr/bin/perl
#
# $Header: //sapdb/V76/develop/sys/src/install/perl/SAPDB/Install/Dialog/Prepare.pm#1 $
# $DateTime: 2004/03/11 10:29:37 $
# $Change: 66919 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade,
# called by SDBINST/SDBUPD
#

package SAPDB::Install::Wizard::Dialog::SoftwareParameter;

use Wx qw (
	wxRA_SPECIFY_COLS wxALL wxBOTTOM
	wxVERTICAL wxEXPAND wxTOP
	wxICON_ERROR wxCENTRE);

use base SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::Migrate::Registry;
use SAPDB::Install::System;
use SAPDB::Install::Window::Progress;
use SAPDB::Install::Window::Parameter;
use SAPDB::Install::Prepare;
use SAPDB::Install::Tools;
use SAPDB::Install::Registry;

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->SetSizer(Wx::BoxSizer->new(wxVERTICAL));	
	$this->SetAutoLayout(1);
	$this->{'show_self_defined'} = 1;
	return $this;
}

sub init{
	my ($self,$config) = @_;
	$self->{'current_package'} = undef;
	
	#
	# dirty hack: initialize indep_path cache
	# to get Base package initialized properly
	# if indep_data is already registerd
	# but indep_prog not    
	#
	readIndepPath();
	#######################################
	
	return {'auto_next' => 1};
}


sub ParentSelection{
		my ($self,$config,$parents) = @_;
		my @line;
		my $parent_name;
		my $package = $config->{'packages'}->[$self->{'current_package'}];
		$self->{'parent_selection_pendig'} = 1;
		$self->{'parents'} = $parents;
		foreach my $parent_id (@$parents){
			$parent_name = $parent_id->[0];
			my $parent_path = $parent_id->[1];
			$parent_path = getRealPathName($parent_path)if $^O =~ /mswin/i;
			my $version = $config->{'collector'}->{'regpackage_index'}->{$parent_id->[0]}->{$parent_id->[1]}->{'SOFTWARE_VERSION'};
			push @line, $parent_path.' ['.$version.']';	
		}
		$self->{'parent_rbox'} = Wx::RadioBox->new($self,-1,'Select an existing '.$parent_name.' using '.$package->DispName ,[150,130],[300,200],\@line,1,wxRA_SPECIFY_COLS);
		unless(Wx::wxGTK){
			$self->{'parent_rbox'}->SetSize($self->{'parent_rbox'}->GetBestSize);
		}
		$self->GetSizer->Add($self->{'parent_rbox'},1,wxEXPAND |  wxALL, 80);
		$self->Layout;
		$self->GetWizard->LayoutDlg;		
	return 1;
}


sub ProcessParentSelection{
	my ($self,$config) = @_;
	my $package = $config->{'packages'}->[$self->{'current_package'}];
	my $selection = $self->{'parent_rbox'}->GetSelection;	
	my $parent = $self->{'parents'}->[$selection];
	$package->ParentPackage($parent);
	$package->MainPath->{'value'} = $parent->[1];
	delete $self->{'parents'};
	delete $self->{'parent_selection_pendig'};
	$self->GetSizer->Remove($self->{'parent_rbox'});
	$self->{'parent_rbox'}->Destroy;
	delete $self->{'parent_rbox'};
	return 1;
}

sub UpdateSelection{
	my ($self,$config,$update_infos) = @_;
	my $package = $config->{'packages'}->[$self->{'current_package'}];
	my @line;
	$self->{'to_update'} = $update_infos->{'toupdate'};
	$self->{'update_sb'} = Wx::StaticBox->new($self,-1,'Select an existing '.$package->DispName);
	$self->{'update_sbsizer'} = Wx::StaticBoxSizer->new($self->{'update_sb'},wxVERTICAL); 
	my @update_rbs = ();
	foreach my $inst (sort {$a->[0] <=> $b->[0]} @{$update_infos->{'toupdate'}}){
		my $path = $inst->[0];
		if($^O =~ /mswin/i){
			$path = getRealPathName($path);
		}
		
		my $label = $path.
				(	$inst->[3] ? 
					'  ('.$inst->[1].
					(defined $inst->[2] ? ' '.$inst->[2].'bit' : '')
					.')'
				: 
					'  (flagged as invalid)') ;
		
		my $rb = Wx::RadioButton->new($self,-1,$label);
		$self->{'update_sbsizer'}->Add($rb,0,(wxALL) & (~wxBOTTOM),5);
		push @update_rbs,$rb;
	}
	my $rb = Wx::RadioButton->new($self,-1,'None  (New Installation)');
	$self->{'update_sbsizer'}->Add($rb,0,(wxALL) & (~wxBOTTOM),5);	
	push @update_rbs,$rb;
	$self->{'update_rbs'} = \@update_rbs;
	$self->GetSizer->Add($self->{'update_sbsizer'},1,wxEXPAND |  wxALL, 80);
	$self->Layout;
	$self->GetWizard->LayoutDlg;
	$self->{'update_selection_pending'} = 1;
 	return 1;	
}


sub ShowProgress{
	my ($self,$enable,$config) = @_;
	if($enable){
		return if $self->{'progress_shown'};
		my $package = $config->{'packages'}->[$self->{'current_package'}];
		if(defined $self->{'progresswin'}){
			$self->{'progresswin'}->Show(1);
		}
		else{		
			$self->{'progresswin'} = SAPDB::Install::Window::Progress->new($self,-1);
		}
		if(defined $self->{'static_box'}){
			$self->{'static_box'}->SetLabel('Doing update checks of package '.$package->DispName);
			$self->{'static_box'}->Show(1);
		}
		else{
			$self->{'static_box'} = Wx::StaticBox->new($self,-1,
									'Doing update checks of package '.$package->DispName);
		}
		$self->{'box_sizer'} = Wx::StaticBoxSizer->new($self->{'static_box'},wxVERTICAL); 
		$self->{'progresswin'}->Fit;
		$self->{'box_sizer'}->Add($self->{'progresswin'},1,wxEXPAND | wxALL,40);
		$self->{'box_sizer'}->CalcMin;
		$self->GetSizer->Add($self->{'box_sizer'},1,wxEXPAND | wxALL,60);
		$self->{'progress_shown'} = 1;
		$self->GetWizard->LayoutDlg;
		$self->Layout;
	}
	else{
		return unless $self->{'progress_shown'};
		$self->{'box_sizer'}->Remove($self->{'progresswin'});
		$self->{'progresswin'}->Show(0);
		$self->GetSizer->Remove($self->{'box_sizer'});
		$self->{'static_box'}->Show(0);
		delete $self->{'box_sizer'};
		delete $self->{'progress_shown'};
	}	
}



sub ProcessUpdateSelection{
	my ($self,$config) = @_;
	my $package = $config->{'packages'}->[$self->{'current_package'}];
	
	my $selection = 0;
	foreach my $rb (@{$self->{'update_rbs'}}){
		last if($rb->GetValue);
		$selection++;	
	}
	
	my $new_installation = 0;
	if($selection <= $#{@{$self->{'to_update'}}}){
		$package->MainPath->{'value'} = $self->{'to_update'}->[$selection]->[0];		
		$config->{'container'}->Log->SetMsg("MSG: select update of installation in ".$package->MainPath->{'value'}."\n");	
		$package->Update(1);
	}
	else{
		# new installation
		$config->{'container'}->Log->SetMsg("MSG: select new installation\n");
		$new_installation = 1;
	}
	delete $self->{'update_selection_pending'};
	delete $self->{'to_update'};
	$self->GetSizer->Remove($self->{'update_sbsizer'});
	delete $self->{'update_sbsizer'};
	foreach my $rb (@{$self->{'update_rbs'}}){
		$rb->Destroy;
	}
	delete $self->{'update_rbs'};
	$self->{'update_sb'}->Destroy;
	delete $self->{'update_sb'};
	if($new_installation){
		$self->NewInstallation($config);
		return undef;
	}
	return 1;
}

sub NewInstallation{
	my ($self,$config) = @_;
	my $package = $config->{'packages'}->[$self->{'current_package'}];
	$self->{'static_box'} = Wx::StaticBox->new($self,-1,"Settings of Package ".$package->DispName);
	$self->{'box_sizer'} = Wx::StaticBoxSizer->new($self->{'static_box'},wxVERTICAL); 
	$self->{'paramwin'} = SAPDB::Install::Window::Parameter->new($self,-1,[0,0],[100,100]);
	my %params;
	my $seq = 0;
	if($self->{'owner_unknown'}){
		$params{'owner'}->{'type'} = 'string';
		$params{'owner'}->{'label'} = 'Software Owner';
		$params{'owner'}->{'default'} = 'sdb';
		$params{'owner'}->{'seq'} = $seq++;
	}
	if($self->{'group_unknown'}){		
		$params{'group'}->{'type'} = 'string';
		$params{'group'}->{'label'} = 'Administrators Group';
		$params{'group'}->{'default'} = 'sdba';
		$params{'group'}->{'seq'} = $seq++;			
	}
	foreach my $path (@{$package->Paths}){
		next if (defined $path->{'value'});
		$params{$path->{'name'}}->{'type'} = 'path';
		my $label = $path->{'name'};
		$label =~ s/^(\w)/\u$1/;
		$label =~ s/(\s)(\w)/$1\u$2/g;
		$params{$path->{'name'}}->{'label'} = $label;
		$params{$path->{'name'}}->{'seq'} = $seq++;
		$params{$path->{'name'}}->{'default'} = $^O !~ /mswin/i ? $path->{'default'} : getRealPathName($path->{'default'});
	}
	$self->{'paramwin'}->init(\%params);
	$self->{'box_sizer'}->Add($self->{'paramwin'},1,wxEXPAND | wxALL ,20);	
	$self->GetSizer->Add($self->{'box_sizer'},1,wxEXPAND | wxALL ,60);
	$self->Layout;
	$self->{'newinstallation_pending'} = 1;				
	$self->GetWizard->LayoutDlg;
	$self->Layout;
	return 1;
}


sub ProcessNewInstallation{
	my ($self,$config) = @_;
	my $package = $config->{'packages'}->[$self->{'current_package'}];
	my $params = $self->{'paramwin'}->SetValues();
	my $errstate = 0;
	
	foreach my $param (keys(%$params)){
		if($params->{$param}->{'type'} eq 'path'){
			if($param eq $package->MainPath->{'name'}){
				if(defined $config->{'container'}->InstReg and 
				$config->{'container'}->InstReg->existPackage($package->Name,
												normalizePath($params->{$param}->{'value'}))
					){
					$self->SetError("Please select another installation path!","Installation in ".$params->{$param}->{'value'}." already exists!");
					return undef;
				}
				$package->MainPath->{'realname'} = getRealPathName($params->{$param}->{'value'}) if($^O =~ /mswin/i);
				$package->MainPath->{'value'} = normalizePath($params->{$param}->{'value'});
			}
			my $found = 0;
			foreach my $path (@{$package->Paths}){
				if($path->{'name'} eq $param){
					$path->{'value'} = normalizePath($params->{$param}->{'value'});
					$path->{'realname'} = getRealPathName($params->{$param}->{'value'}) if($^O =~ /mswin/i);
					$found = 1;
					last;
				}
			}
			unless($found){
				my $msgbox = Wx::MessageDialog->new($self,"path parameter \"$param\" not found",
					"Error: parameter not found", wxICON_ERROR | wxCENTRE);
				$msgbox->ShowModal();
				$errstate = 1;
				last;
			}
			$config->{'container'}->Log->SetMsg("MSG: selected new installation in \"".$package->MainPath->{'value'}."\"\n");
		}
		else{
			if($self->{'owner_unknown'}){
				if($param eq 'owner'){
					$SAPDB::Install::Values::owner = $params->{$param}->{'value'};
					$package->User($params->{$param}->{'value'});	
					delete $self->{'owner_unknown'};
				}	
			}
			if($self->{'group_unknown'}){
				if($param eq 'group'){
						$SAPDB::Install::Values::group = $params->{$param}->{'value'};
						$package->Group($params->{$param}->{'value'});	
						delete $self->{'group_unknown'};
				}
			}
		}
	}
	if($errstate){
		return undef;
	}	
	$package->Update(0);
	$self->{'box_sizer'}->Remove($self->{'paramwin'});
	$self->{'paramwin'}->Destroy;
	delete $self->{'paramwin'};
	$self->GetSizer->Remove($self->{'box_sizer'});
	delete $self->{'box_sizer'};
	$self->{'static_box'}->Destroy;
	delete $self->{'static_box'};
	delete $self->{'newinstallation_pending'};
	return 1;
}

sub DoPackage{
	my ($self,$config) = @_;
	my $package = $config->{'packages'}->[$self->{'current_package'}];

	if($self->{'update_selection_pending'}){
		return $self->ProcessUpdateSelection($config);
		
	}
	elsif($self->{'newinstallation_pending'}){
		return $self->ProcessNewInstallation($config);
	}
	elsif($self->{'parent_selection_pendig'}){
		$self->ProcessParentSelection($config);
	}

	if(defined $package->preinstall){
		$config->{'container'}->Log->SetMsg("MSG: calling preinstall...\n");
		eval{
			&{$package->preinstall}(\$config->{'container'}->InstReg);
		};
		if($@){
			die($@);
		}		
	}
	
	#
	#	owner/group settings
	#

	unless($^O =~ /mswin/i){
		if($package->Name eq 'Base'){
			if($SAPDB::Install::Values::owner =~ /\S/){
				$package->User($SAPDB::Install::Values::owner);
			}
			elsif(defined $config->{'container'}->Template and 
				defined $config->{'container'}->Template->getOwnerName){
				$package->User($config->{'container'}->Template->getOwnerName);
				$SAPDB::Install::Values::owner = $config->{'container'}->Template->getOwnerName;	
			}
			else{
				$self->{'owner_unknown'} = 1;	
			}
			if($SAPDB::Install::Values::group =~ /\S/){
				$package->Group($SAPDB::Install::Values::group);
			}
			elsif(defined $config->{'container'}->Template and 
				defined $config->{'container'}->Template->getOwnerGroup){
				$package->Group($config->{'container'}->Template->getOwnerGroup);
				$SAPDB::Install::Values::group = $config->{'container'}->Template->getOwnerGroup; 	
			}
			else{
				$self->{'group_unknown'} = 1;			
			}
		}
		else{
			$package->User($SAPDB::Install::Values::owner);
			$package->Group($SAPDB::Install::Values::group);	
		}

	}

	my $paths_ok = 1;
	foreach my $path (@{$package->Paths}){
		unless(defined $path->{'value'}){
			$paths_ok = 0;
			last;
		}
	}

	if($package->MainPath->{'value'} =~ /\S/ and $paths_ok){
		$config->{'container'}->Log->SetMsg("MSG: path was set by preinstall script (".
													$package->MainPath->{'value'}.")\n");
	}
	elsif($package->Name eq 'Base' and 
		defined $config->{'container'}->Template){
		foreach my $path (@{$package->Paths}){
			next if $path->{'value'} =~ /\S/;
			if($path->{'name'} eq 'independent data path'){
				my $indep_data = $config->{'container'}->Template->getIndepData;
				$path->{'realname'} = getRealPathName($indep_data) if $^O =~ /mswin/i;
				normalizePath(\$indep_data);
				next unless $indep_data =~ /\S/;
				$config->{'container'}->Log->SetMsg("MSG: Template set indep data path to \"$indep_data\"\n");
				$path->{'value'} = $indep_data;
				$SAPDB::Install::Values::indep_data_path = $indep_data;
			}
			elsif($path->{'name'} eq 'independent program path'){
				my $indep_prog = $config->{'container'}->Template->getIndepProg;
				next unless $indep_prog =~ /\S/;
				$path->{'realname'} = getRealPathName($indep_prog) if $^O =~ /mswin/i;
				normalizePath(\$indep_prog);
				$config->{'container'}->Log->SetMsg("MSG: Template set indep programs path to \"$indep_prog\"\n");
				$path->{'value'} = $indep_prog;
				$package->MainPath->{'value'} = $indep_prog;
				$package->MainPath->{'realname'} = $path->{'realname'} if $^O =~ /mswin/i;
				$SAPDB::Install::Values::indep_prog_path = $indep_prog;
			}
		}
	}
	elsif($package->Name eq 'Database Kernel' and 
		(defined $config->{'container'}->Template or
		defined $config->{'dbupdate'})){
		my $got_path;
		
		if(defined $config->{'container'}->Template){
			$got_path = $config->{'container'}->Template->getDBInstance()->getDependendPath;
		}
		else{
			$got_path = $config->{'dbupdate'}->{'instroot'};
		}
				
		foreach my $path (@{$package->Paths}){
			if($path->{'name'} eq 'dependent path'){
				my $depend = $got_path;
				$path->{'realname'} = getRealPathName($depend) if $^O =~ /mswin/i;
				normalizePath(\$depend);
				next unless $depend =~ /\S/;
				$config->{'container'}->Log->SetMsg("MSG: Template or instance update set dependent path to $depend\n");
				$path->{'value'} = $depend;
				$package->MainPath->{'value'} = $depend;
				$package->MainPath->{'realname'} = $path->{'realname'}  if $^O =~ /mswin/i;
			}
		}
	}
	else{
		my $update_infos = collectUpdateInfos($package,$config->{'packages'},
												\($config->{'container'}->InstReg))
												if defined $config->{'container'}->InstReg;
		
		if($package->IsSubPackage){
			if(defined $package->ParentPackage){
					#
					#	parent package already set 
					#	there is nothing to be done
			}
			elsif(defined $update_infos->{'possible_parents'}){
				
				#
				#	select a parent package
				#
				my @possible_parents = @{$update_infos->{'possible_parents'}};
				if($#possible_parents == 0){
					$package->ParentPackage($possible_parents[0]);
					$package->MainPath->{'value'} = $possible_parents[0]->[1];
				}
				else{
					$self->ParentSelection($config,$update_infos->{'possible_parents'});			
					return undef;
				}
			}
			else{
				
				#
				#	no parent package available
				#
				$self->SetError('Dependency Error','Parent package not found');
			
				return undef;
			}
		}
		else{
			if(defined $update_infos->{'toupdate'} and $#{@{$update_infos->{'toupdate'}}} > -1){
				$self->UpdateSelection($config,$update_infos);
				return undef;
			}
			else{
				$self->NewInstallation($config);	
				return undef;
			}
		}	
	}

	if(defined $config->{'container'}->InstReg){
		if($config->{'container'}->InstReg->
			existPackage($package->Name,$package->MainPath->{'value'})){
			$package->Update(1);
		}
		else{
			$package->Update(0);
		}
	}
	else{
		$package->Update(0);
	}	

	return 1;
}






sub initPackage{
	my ($self,$config) = @_;
	my $package = $config->{'packages'}->[$self->{'current_package'}];
	unless ($^O =~ /mswin/i){
		if($package->Name eq 'Base'){
			$SAPDB::Install::Values::group = $package->Group;
			$SAPDB::Install::Values::owner = $package->User;
		}

		my $gid = getgrnam($package->Group);
		
		if($gid eq ''){
			$gid = createGroup($package->Group);
		}
		
		$package->GID($gid);
		
		my $uid = getpwnam($package->User);
		
		if($uid eq ''){
			$uid = createSDBUser($package->User,$package->Group);
		}
		
		$package->UID($uid);
		
		
	}
	
	if($package->Name eq 'Base' and not defined $config->{'container'}->InstReg){
		my $data_path;
		foreach my $path (@{$package->Paths}){
			if($path->{'name'} eq 'independent data path'){
				$SAPDB::Install::Values::indep_data_path = $path->{'value'}; 
				$data_path = $path->{'value'};
				$data_path =  $path->{'realname'} if $^O =~ /mswin/;
			}
			elsif($path->{'name'} eq 'independent program path'){
				$SAPDB::Install::Values::indep_prog_path = $path->{'value'}; 
			}
		}
		$config->{'container'}->Log->LogPath("$data_path/wrk");
		makedir("$data_path/config",0775,$package->UID,$package->GID);
		makedir("$data_path/config/install",0775);
		unless(-f "$data_path/config/install/INSTREG"){
			$self->{'registry_created'} = 1;
		}
		$config->{'container'}->InstReg(SAPDB::Install::InstallRegistry->
			new("$data_path/config/install",1,$config->{'container'}->Log));
		
		$SAPDB::Install::StartInstall::instRegistry = $config->{'container'}->InstReg;
		$config->{'container'}->SetRefInstReg(\$SAPDB::Install::StartInstall::instRegistry);
		
		migrateReg($config->{'container'}->InstReg,@{$config->{'packages'}});
		
		if($config->{'container'}->InstReg->
			existPackage($package->Name,$package->MainPath->{'value'})){
			$package->RegData($config->{'container'}->
				InstReg->getPackage($package->Name,$package->MainPath->{'value'}));
			$package->Update(1);
		}
	}
	
	return 1;
		
}

sub Process{
	my ($self,$config) = @_;
	my $index = 0;
	
	#
	#	test if indep_prog is set in RTE registry.
	#	use InstallRegistry value if there is no such RTE entry
	#
	if(	defined $config->{'container'}->InstReg and 
		$SAPDB::Install::Values::indep_prog_path !~ /\S/){
		my %base_paths = $config->{'container'}->InstReg()->getInstallPathes('Base');
		my @base_paths = keys(%base_paths);
		if($#base_paths == 0){
			if(defined $config->{'container'}->Template){
				$config->{'container'}->Template->setIndepProg($base_paths[0]);
			}
			$SAPDB::Install::Values::indep_prog_path = $base_paths[0];
		}
	}
	
	#
	#	iterate packages
	#

	foreach my $package (@{$config->{'packages'}}){
		++$index and next if defined $self->{'current_package'} and $index < $self->{'current_package'}; 
		$self->{'current_package'} = $index++;
		my $rc = $self->DoPackage($config);
		return(undef) unless defined $rc;
		return $rc if exists $rc->{'errstate'};
		next if (${($package->Skip)});
		
		my %ext_args = ('callback_progress' => sub {$self->{'progresswin'}->SetProgress(@_)},
						'callback_initprogress' => sub {$self->{'progresswin'}->init(@_)});
		
		unless($^O =~ /mswin/i){
			$ext_args{'opt_g'} = $package->Group;
			$ext_args{'opt_o'} = $package->User;
			$ext_args{'opt_b'} = 1;
		}

		if($package->Update){
			
			$self->ShowProgress(1,$config);
			my ($rc,$details) = checkUpdate($package,$config->{'packages'},\$config->{'container'}->InstReg,\%ext_args);
			if($rc == 0){
				my $top_message = 'Update Check Failed!';
				$self->SetError("Error: $top_message",$details->{'errmsg'});
				return {'errstate' => 1};				
			}		
			$self->ShowProgress(0);
			next if (${($package->Skip)});
		}
		$self->initPackage($config);
		if($package->Name eq 'Base' and $self->{'registry_created'} and
			$config->{'container'}->InstReg->existPackage('Base',$package->MainPath->{'value'})){
			$package->Update(1);
			$self->ShowProgress(1,$config);
			my ($rc,$details) = checkUpdate($package,$config->{'packages'},\$config->{'container'}->InstReg,\%ext_args);
			if($rc == 0){
				my $top_message = 'Update Check Failed!';
				$self->SetError("Error: $top_message",$details->{'errmsg'});
				return {'errstate' => 1};				
			}		
			$self->ShowProgress(0);
			next if (${($package->Skip)});
		} 
	}	
	return 1;
}


sub Back{
	my ($self,$config) = @_;
	foreach my $package (@{$config->{'packages'}}){
		my $skip = 0;
		$package->Skip(\$skip);
		$package->Update(undef);
		$package->ParentPackage(undef);		
		$package->MainPath->{'value'} = undef;
		foreach my $path (@{$package->Paths}){
			$path->{'value'} = undef;
			if(defined $package->RegData){
				$SAPDB::Install::InstallRegistry::Package::SELFOBJ[$package->RegData->ID] = undef;
				$package->RegData(undef);
			}
		}
	}
	if($self->{'registry_created'}){
		$config->{'container'}->InstReg(undef);
		my $instreg_dir = $SAPDB::Install::Values::indep_data_path.'/config/install';
		if(-d $instreg_dir){
			if(opendir(DH,$instreg_dir)){
				my @files = grep {-f "$instreg_dir/$_" && /INSTREG/} readdir(DH);
				closedir(DH);  
				foreach my $regfile (@files){
					unlink("$instreg_dir/$regfile");
				}
				removeEmptyDirs($SAPDB::Install::Values::indep_data_path);
			}
		}
		delete $self->{'registry_created'}; 	
	}
	$SAPDB::Install::Values::indep_data_path = undef;
	$SAPDB::Install::Values::indep_prog_path = undef;
	if($self->{'newinstallation_pending'}){
		$self->{'box_sizer'}->Remove($self->{'paramwin'});
		$self->{'paramwin'}->Destroy;
		delete $self->{'paramwin'};
		$self->GetSizer->Remove($self->{'box_sizer'});
		delete $self->{'box_sizer'};
		$self->{'static_box'}->Destroy;
		delete $self->{'static_box'};
		delete $self->{'newinstallation_pending'};
	}
	
	if($self->{'update_selection_pending'}){
		delete $self->{'to_update'};
		$self->GetSizer->Remove($self->{'update_sbsizer'});
		delete $self->{'update_sbsizer'};
		foreach my $rb (@{$self->{'update_rbs'}}){
			$rb->Destroy if defined $rb;
		}
		delete $self->{'update_rbs'};
		$self->{'update_sb'}->Destroy if defined $self->{'update_sb'};
		delete $self->{'update_sb'};
		delete $self->{'update_selection_pending'};
	}
	
	if($self->{'parent_selection_pendig'}){
		$self->GetSizer->Remove($self->{'parent_rbox'});
		$self->{'parent_rbox'}->Destroy;
		delete $self->{'parent_rbox'};
		delete $self->{'parents'};
		delete $self->{'parent_selection_pendig'};
	}
	
	delete $self->{'current_package'};
	return 1;
}


sub BackInit{
	my ($self,$config) = @_;
	return {'auto_back' => 1};
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer->Remove(0)){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'progresswin'}){
		$self->{'progresswin'}->Destroy;
		delete $self->{'progresswin'};
	}
	if(defined $self->{'update_rbs'}){
		foreach my $rb (@{$self->{'update_rbs'}}){
			$rb->Destroy if defined $rb;
		}
		delete $self->{'update_rbs'};
	}
	if(defined $self->{'update_sb'}){
		$self->{'update_sb'}->Destroy;
		delete $self->{'update_sb'};
	}
	if(defined $self->{'parent_rbox'}){
		$self->{'parent_rbox'}->Destroy;
		delete $self->{'parent_rbox'};
	}
	if(defined $self->{'static_box'}){
		$self->{'static_box'}->Destroy;
		delete $self->{'static_box'};
	}
	if(defined $self->{'paramwin'}){
		$self->{'paramwin'}->Destroy;
		delete $self->{'paramwin'};

	}

	$self->SUPER::Destroy;
}



sub DESTROY{
	my ($self) = @_;
	print ">>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
