#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/UpdateFinish.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc:
#

package SAPDB::Install::Wizard::Dialog::UpdateFinish;

use Wx::Event qw(EVT_TEXT);

use Wx qw (	wxVERTICAL wxHORIZONTAL wxEXPAND
			wxCB_READONLY wxALL wxTOP wxRIGHT
			wxLC_REPORT wxTE_READONLY wxTE_MULTILINE);


use SAPDB::Install::Tools;
use base SAPDB::Install::Wizard::Dialog::Base;

sub new{
	my($this ) = shift->SUPER::new(@_);
	return $this;
}

sub init(){
	my ($self,$config) = @_;
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	$self->{'label'} = Wx::StaticText->new($self,-1,'Finishing database update of '.$config->{'dbupdate'}->{'dbname'});
	$self->{'text'} = Wx::TextCtrl->new($self,-1,'',[0,0],[150,100],wxTE_READONLY | wxTE_MULTILINE);
	$sizer->Add($self->{'label'},0,wxALL,20);
	$sizer->Add($self->{'text'},1,wxEXPAND | wxALL,20);
	$self->SetAutoLayout(1);
	$self->SetSizerAndFit($sizer);
	$self->{'initialized'} = 1;
	$self->GetWizard()->KillPrevSteps();
	return {'auto_next' => 1}; 
}

sub Process{
	my ($self,$config) = @_;

	$config->{'container'}->Log->SetMsg("MSG: starting new instance check\n");
	
	my $instance_data = $config->{'dbupdate'}->{'registry_instancedata'};
	
	require SAPDB::Install::Instance;
	my $instance = SAPDB::Install::Instance->new ();
	
	$instance->{'msg0'}= sub {
			my ($text) = @_;
			$self->{'text'}->AppendText($text);
			&{$SAPDB::Install::Values::RefreshGui}();
	};
	
	$instance->{'msg1'}= 
		sub {
			$config->{'container'}->Log->SetMsg(@_);
			&{$SAPDB::Install::Values::RefreshGui}(); 
		};
	
	my $release = $config->{'collector'}->package_index->
				  {'Database Kernel'}->{'SOFTWARE_VERSION'};
				  
	$instance->chvolperm($config->{'dbupdate'}->{'dbname'}, $instance_data->{'STARTING_RELEASE'}, $release);
	
	if($instance->get_errorstate () ne 'OK'){
		$self->{'text'}->AppendText('ERROR: '.$instance->{msgtxt});
		$self->SetError('cannot finish instance update for '.$config->{'dbupdate'}->{'dbname'},
							$instance->{'msgtxt'});
		return undef;
	}
	
	$config->{'comm'}->refresh();
	
	unless($config->{'comm'}->start){
		$self->SetError('Cannot start communication server',$config->{'comm'}->last_error);
		return undef;		
	}

	my $rc = $instance->session (
	'apo_target_release' => $instance_data->{'APO_TARGET_RELEASE'},
	'apo_starting_release' => $instance_data->{'APO_STARTING_RELEASE'},	
	'dbname' => $config->{'dbupdate'}->{'dbname'},
	'dbhost' => 'localhost', 
	'userpwd' => $config->{'dbupdate'}->{'user_passwd'},
	'target_release' => $release,
	'instancetype' => $instance_data->{'TYPE'},
	'starting_release' => $instance_data->{'STARTING_RELEASE'},
	'migration_strategy' => $instance_data->{'MIGRATION_STRATEGY'},
	'default_code' => $instance_data->{'DEFAULT_CODE'},
	'mcod' => $instance_data->{'MCOD'},
	'lcowner' => $instance_data->{'LCOWNER'});
	
	
	unless($rc){
		$self->SetError('connection failed','cannot establish database connection');
		return undef;
	}
	
	$instance->finalize();
	
	if($instance->get_errorstate () ne 'OK'){
		$self->{'text'}->AppendText('ERROR: '.$instance->{msgtxt});
		$self->SetError('cannot finish instance update for '.$config->{'dbupdate'}->{'dbname'},
							$instance->{'msgtxt'});
		return undef;
	}
	
	if($instance->get_keep_instance_data){
		
	}
	else{
		if($config->{'container'}->InstReg->DenyWrite){
			$config->{'container'}->InstReg->DenyWrite(0);
		}
		$config->{'container'}->InstReg->removeInstanceData($config->{'dbupdate'}->{'dbname'});
	}			  
	return 1;
}

sub BackInit{
	my ($self,$config) = @_;
	return 1;
}
	

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
